package webrtc

// Signal describes a message that will be sent via signalling channel to discover peers.
type Signal struct {
	UID  string // optional user identifier to distinguish peers
	Data []byte // payload generated by WebRTC to establish connection
}

// AnswerStream is stream of responses to a discovery request.
type AnswerStream interface {
	Next() (Signal, error)
	Close() error
}

// Offer is a discovery message from a specific peer.
type Offer interface {
	Info() Signal
	Answer(s Signal) error
}

// OfferStream is a stream of discovery messages coming from other peers.
type OfferStream interface {
	Next() (Offer, error)
	Close() error
}

// Signalling is an interface for a signalling connection that helps to establish P2P connections.
type Signalling interface {
	// Broadcast sends a peer discovery request.
	Broadcast(s Signal) (AnswerStream, error)
	// Listen starts listening for incoming peer discovery requests.
	Listen(uid string) (OfferStream, error)
}
